               JOB  1401 RANDMU
               CTL  6111
     *
     **************************************
     * 2-DIGIT RANDOM NUMBERS             *
     * UNIFORMLY AND NORMALLY DISTRIBUTED *
     *                                    *
     * BY RONALD MAK                      *
     * NOVEMBER 23, 2019                  *
     **************************************
     *
     * INDEX REGISTERS
               ORG  87
     X1        DCW  @004@  POSITION IN THE PRINT AREA
               ORG  92
     X2        DCW  @000@  INDEX FOR THE UNIFORM FREQUENCY TABLE
               ORG  97
     X3        DCW  @000@  INDEX FOR THE NORMAL FREQUENCY TABLE  
               ORG  350
     *
     * TWO 7-DIGIT SEEDS FOR RANDOM NUMBER GENERATION
     R1        DCW  @1234567@  FIRST SEED
               DC   @XXX@      SPILL AREA FOR RIGHT ROTATE 3
     R2        DCW  @8901234@  SECOND SEED
               DC   @XXXXX@    SPILL AREA FOR RIGHT ROTATE 5
     *
     RRSUM     DCW  @0000000@  SUM OF R1 AND R2
     RN        EQU  RRSUM-5    RANDOM NUMBER (FIRST 2 DIGITS OF RRSUM)
     SSUM      DCW  @0000@     SAMPLE SUM
     SAVG      EQU  SSUM-2     SAMPLE AVERAGE (FIRST 2 DIGITS OF SSUM)
     SSIZE     DCW  @000@      SAMPLE SIZE
     COUNTR    DCW  @000@      COUNTER
     *
               ORG  400
     START     EQU  *&1
     *
     * ZERO OUT THE UNIFORM FREQUENCY TABLE
     NXTUZ     MCW  @000@,UFREQ&X2
               SBR  X2,3&X2
               C    @300@,X2
               BU   NXTUZ
     *
     * ZERO OUT THE NORMAL FREQUENCY TABLE
     NXTNZ     MCW  @000@,NFREQ&X3
               SBR  X3,3&X3
               C    @300@,X3
               BU   NXTNZ
     *
     *-----------------------------------------------------
     * TABLE OF 10,000 UNIFORMLY DISTRIBUTED RANDOM NUMBERS
     *-----------------------------------------------------
     *
               MCW  @2-DIGIT RANDOM NUMBERS@,296
               MCW  @TABLE OF 10,000 UNIFORMLY DISTRIBUTED @
               W
               CS   332
               CS
               W
     *
     * NXTRN LOOP TO GENERATE THE NEXT RANDOM NUMBER
     *
     NXTRN     MCW  R1,R1&3       RIGHT ROTATE R1 IN PLACE BY 3
               SW   R1&1
               MCW  R1&3,R1-4
               CW   R1&1
     * 
               MCW  R2,R2&5       RIGHT ROTATE R2 IN PLACE BY 5
               SW   R2&1
               MCW  R2&5,R2-2
               CW   R2&1
     *
               MCW  R2,RRSUM      ADD R1 AND R2
               A    R1,RRSUM
               MZ   @0@,RRSUM-6   CLEAR OVERFLOW
     *
               MCW  R2,R1         R1 = R2
               MCW  RRSUM,R2      R2 = RRSUM
               MCW  RN,203&X1     MOVE RANDOM NUMBER TO THE PRINT AREA
     *
     * UPDATE THE UNIFORM FREQUENCY TABLE
               MCW  @000@,X2
               MCW  RN,X2
               A    RN,X2
               A    RN,X2
               A    @001@,UFREQ&X2
     *
     * UPDATE THE SAMPLE SUM
               A    RN,SSUM
               A    @001@,SSIZE
               C    @100@,SSIZE
               BL   NXTPR
     *
     * UPDATE THE NORMAL FREQUENCY TABLE
               MCW  @000@,X3
               MCW  SAVG,X3
               A    SAVG,X3
               A    SAVG,X3
               A    @001@,NFREQ&X3
               MCW  @0000@,SSUM
               MCW  @000@,SSIZE
     *
     * UPDATE X1 TO PRINT THE NEXT RANDOM NUMBER
     NXTPR     SBR  X1,3&X1
               C    @064@,X1      MIDDLE OF THE LINE?
               BU   CHKEOL
               SBR  X1,4&X1       ADD A GAP IN THE MIDDLE
     CHKEOL    C    @128@,X1      END OF THE LINE?
               BU   NXTRN
     *
     * WRITE ONE LINE
               W
               CS   332
               CS
     *
     * PREPARE FOR THE NEXT LINE
               MCW  @004@,X1
               A    @001@,COUNTR
               C    @250@,COUNTR  250 LINES PRINTED?
               BU   NXTRN
     *
     *----------------------------------------------------------------
     * FREQUENCY BAR CHART OF THE UNIFORMLY DISTRIBUTED RANDOM NUMBERS
     *----------------------------------------------------------------
     *
               W
               MCW  @RANDOM NUMBERS@,297
               MCW  @FREQUENCY CHART OF 10,000 UNIFORMLY DISTRIBUTED @
               W
               CS   332
               CS
               W
     *
               MCW  @000@,RN
               MCW  @000@,X2
     *
     * NXTUF LOOP TO PRINT THE BAR FOR THE NEXT FREQUENCY VALUE
     *
     NXTUF     MCW  UFREQ&X2,206  FREQUENCY VALUE TO THE PRINT AREA
               MCW  UFREQ&X2,COUNTR
               MCW  @000@,X1
               B    PRBAR
               C    @300@,X2
               BL   NXTUF
     *
     *---------------------------------------------------------------
     * FREQUENCY BAR CHART OF THE NORMALLY DISTRIBUTED RANDOM NUMBERS
     *---------------------------------------------------------------
     *
               W
               MCW  @RANDOM NUMBERS@,258
               MCW  @FREQUENCY CHART OF 100 NORMALLY DISTRIBUTED @
               W
               CS   332
               CS
               W
     *
               MCW  @040@,RN
               MCW  @120@,X2
     *
     * NXTNF LOOP TO PRINT THE BAR FOR THE NEXT FREQUENCY VALUE
     *
     NXTNF     MCW  NFREQ&X2,206  FREQUENCY VALUE TO THE PRINT AREA
               MCW  NFREQ&X2,COUNTR
               MCW  @000@,X1
               B    PRBAR
               C    @180@,X2
               BL   NXTNF
     *
               H
     *
     *------------------------------------------------------------------
     * FREQUENCY BAR SUBROUTINE
     *     PRINT A BAR OF THE BAR CHART
     *     ON ENTRY, INDEX REGISTER X2 AND RANDOM NUMBER RN MUST BE ZERO
     *     AND COUNTR MUST BE THE NUMBER OF STARS TO PRINT
     *------------------------------------------------------------------
     PRBAR     SBR  PRRTN&3       SAVE THE RETURN ADDRESS
     *
               MCW  RN,202        RANDOM NUMBER TO THE PRINT AREA
               MCW  @-@,203       - TO THE PRINT AREA
     *
     * STARS LOOP TO PRINT THE STARS IN A BAR
     *
     STARS     C    X1,COUNTR     ENOUGH STARS?
               BE   WRBAR
               MCW  @*@,208&X1    * TO THE PRINT AREA
     *
               C    @124@,X1      AT THE END OF THE PRINT AREA?
               BE   WRBAR
     *
               SBR  X1,1&X1       X1 TO THE NEXT PRINT POSITION
               B    STARS
     *
     * WRITE A BAR AS A LINE OF STARS 
     WRBAR     W
               CS   332
               CS
     *
     * PREPARE FOR THE NEXT LINE
               A    @001@,RN
               SBR  X2,3&X2
     *
     PRRTN     B    PRRTN         RETURN
     *
     * FREQUENCY TABLE OF THE UNIFORMLY DISTRIBUTED RANDOM NUMBERS
     *
               ORG  1000
     UFTABL    DA   100X3,X2
     UFREQ     EQU  UFTABL&2
     *
     * FREQUENCY TABLE OF THE NORMALLY DISTRIBUTED RANDOM NUMBERS
     *
     NFTABL    DA   100X3,X3
     NFREQ     EQU  NFTABL&2
     *
               END  START
